# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import diagonal_b6.geometry_pb2 as geometry__pb2
import diagonal_b6.features_pb2 as features__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tapi.proto\x12\x03\x61pi\x1a\x0egeometry.proto\x1a\x0e\x66\x65\x61tures.proto\"]\n\x0f\x43ollectionProto\x12#\n\x04keys\x18\x02 \x03(\x0b\x32\x15.api.LiteralNodeProto\x12%\n\x06values\x18\x04 \x03(\x0b\x32\x15.api.LiteralNodeProto\"X\n\tPairProto\x12$\n\x05\x66irst\x18\x01 \x01(\x0b\x32\x15.api.LiteralNodeProto\x12%\n\x06second\x18\x02 \x01(\x0b\x32\x15.api.LiteralNodeProto\">\n\x15ModifiedFeaturesProto\x12%\n\x03ids\x18\x01 \x03(\x0b\x32\x18.features.FeatureIDProto\"l\n\x12\x41ppliedChangeProto\x12*\n\x08original\x18\x01 \x03(\x0b\x32\x18.features.FeatureIDProto\x12*\n\x08modified\x18\x02 \x03(\x0b\x32\x18.features.FeatureIDProto\"\xb8\x01\n\tNodeProto\x12\x10\n\x06symbol\x18\x01 \x01(\tH\x00\x12(\n\x07literal\x18\x02 \x01(\x0b\x32\x15.api.LiteralNodeProtoH\x00\x12\"\n\x04\x63\x61ll\x18\x03 \x01(\x0b\x32\x12.api.CallNodeProtoH\x00\x12\'\n\x07lambda_\x18\x04 \x01(\x0b\x32\x14.api.LambdaNodeProtoH\x00\x12\r\n\x05\x42\x65gin\x18\x06 \x01(\x05\x12\x0b\n\x03\x45nd\x18\x07 \x01(\x05\x42\x06\n\x04node\"\xe9\x04\n\x10LiteralNodeProto\x12\x12\n\x08nilValue\x18\x01 \x01(\x08H\x00\x12\x13\n\tboolValue\x18\x02 \x01(\x08H\x00\x12\x15\n\x0bstringValue\x18\x03 \x01(\tH\x00\x12\x12\n\x08intValue\x18\x04 \x01(\x03H\x00\x12\x14\n\nfloatValue\x18\x05 \x01(\x01H\x00\x12/\n\x0f\x63ollectionValue\x18\x06 \x01(\x0b\x32\x14.api.CollectionProtoH\x00\x12#\n\tpairValue\x18\x07 \x01(\x0b\x32\x0e.api.PairProtoH\x00\x12.\n\x0c\x66\x65\x61tureValue\x18\x08 \x01(\x0b\x32\x16.features.FeatureProtoH\x00\x12%\n\nqueryValue\x18\t \x01(\x0b\x32\x0f.api.QueryProtoH\x00\x12\x32\n\x0e\x66\x65\x61tureIDValue\x18\n \x01(\x0b\x32\x18.features.FeatureIDProtoH\x00\x12*\n\npointValue\x18\x0b \x01(\x0b\x32\x14.geometry.PointProtoH\x00\x12,\n\tpathValue\x18\x0c \x01(\x0b\x32\x17.geometry.PolylineProtoH\x00\x12\x30\n\tareaValue\x18\r \x01(\x0b\x32\x1b.geometry.MultiPolygonProtoH\x00\x12\x35\n\x12\x61ppliedChangeValue\x18\x0e \x01(\x0b\x32\x17.api.AppliedChangeProtoH\x00\x12\x16\n\x0cgeoJSONValue\x18\x0f \x01(\x0cH\x00\x12&\n\x08tagValue\x18\x10 \x01(\x0b\x32\x12.features.TagProtoH\x00\x42\x07\n\x05value\"O\n\rCallNodeProto\x12 \n\x08\x66unction\x18\x01 \x01(\x0b\x32\x0e.api.NodeProto\x12\x1c\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x0e.api.NodeProto\"=\n\x0fLambdaNodeProto\x12\x0c\n\x04\x61rgs\x18\x01 \x03(\t\x12\x1c\n\x04node\x18\x02 \x01(\x0b\x32\x0e.api.NodeProto\"\x1c\n\rKeyQueryProto\x12\x0b\n\x03key\x18\x01 \x01(\t\"0\n\x12KeyValueQueryProto\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"V\n\x0fTypedQueryProto\x12#\n\x04type\x18\x01 \x01(\x0e\x32\x15.features.FeatureType\x12\x1e\n\x05query\x18\x02 \x01(\x0b\x32\x0f.api.QueryProto\"0\n\x0cQueriesProto\x12 \n\x07queries\x18\x01 \x03(\x0b\x32\x0f.api.QueryProto\"\x0f\n\rAllQueryProto\"\x11\n\x0f\x45mptyQueryProto\"F\n\x08\x43\x61pProto\x12$\n\x06\x63\x65nter\x18\x01 \x01(\x0b\x32\x14.geometry.PointProto\x12\x14\n\x0cradiusMeters\x18\x02 \x01(\x01\"#\n\x0eS2CellIDsProto\x12\x11\n\ts2CellIDs\x18\x01 \x03(\x04\"\xf1\x04\n\nQueryProto\x12!\n\x03\x61ll\x18\x01 \x01(\x0b\x32\x12.api.AllQueryProtoH\x00\x12%\n\x05\x65mpty\x18\x02 \x01(\x0b\x32\x14.api.EmptyQueryProtoH\x00\x12\x0f\n\x05keyed\x18\x03 \x01(\tH\x00\x12$\n\x06tagged\x18\x04 \x01(\x0b\x32\x12.features.TagProtoH\x00\x12%\n\x05typed\x18\x05 \x01(\x0b\x32\x14.api.TypedQueryProtoH\x00\x12)\n\x0cintersection\x18\x06 \x01(\x0b\x32\x11.api.QueriesProtoH\x00\x12\"\n\x05union\x18\x07 \x01(\x0b\x32\x11.api.QueriesProtoH\x00\x12&\n\rintersectsCap\x18\x08 \x01(\x0b\x32\r.api.CapProtoH\x00\x12\x35\n\x11intersectsFeature\x18\t \x01(\x0b\x32\x18.features.FeatureIDProtoH\x00\x12/\n\x0fintersectsPoint\x18\n \x01(\x0b\x32\x14.geometry.PointProtoH\x00\x12\x35\n\x12intersectsPolyline\x18\x0b \x01(\x0b\x32\x17.geometry.PolylineProtoH\x00\x12=\n\x16intersectsMultiPolygon\x18\x0c \x01(\x0b\x32\x1b.geometry.MultiPolygonProtoH\x00\x12.\n\x0fintersectsCells\x18\r \x01(\x0b\x32\x13.api.S2CellIDsProtoH\x00\x12-\n\x0emightIntersect\x18\x0e \x01(\x0b\x32\x13.api.S2CellIDsProtoH\x00\x42\x07\n\x05query\"C\n\x1b\x46indFeatureByIDRequestProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\"G\n\x1c\x46indFeatureByIDResponseProto\x12\'\n\x07\x66\x65\x61ture\x18\x01 \x01(\x0b\x32\x16.features.FeatureProto\":\n\x18\x46indFeaturesRequestProto\x12\x1e\n\x05query\x18\x01 \x01(\x0b\x32\x0f.api.QueryProto\"E\n\x19\x46indFeaturesResponseProto\x12(\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x16.features.FeatureProto\"`\n\x16ModifyTagsRequestProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\x12 \n\x04tags\x18\x02 \x03(\x0b\x32\x12.features.TagProto\"L\n\x1bModifyTagsBatchRequestProto\x12-\n\x08requests\x18\x01 \x03(\x0b\x32\x1b.api.ModifyTagsRequestProto\"\x1e\n\x1cModifyTagsBatchResponseProto\"H\n\x14\x45valuateRequestProto\x12\x1f\n\x07request\x18\x01 \x01(\x0b\x32\x0e.api.NodeProto\x12\x0f\n\x07version\x18\x02 \x01(\t\"7\n\x15\x45valuateResponseProto\x12\x1e\n\x06result\x18\x01 \x01(\x0b\x32\x0e.api.NodeProto2G\n\x02\x42\x36\x12\x41\n\x08\x45valuate\x12\x19.api.EvaluateRequestProto\x1a\x1a.api.EvaluateResponseProtoB\x19Z\x17\x64iagonal.works/b6/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'api_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\027diagonal.works/b6/proto'
  _COLLECTIONPROTO._serialized_start=50
  _COLLECTIONPROTO._serialized_end=143
  _PAIRPROTO._serialized_start=145
  _PAIRPROTO._serialized_end=233
  _MODIFIEDFEATURESPROTO._serialized_start=235
  _MODIFIEDFEATURESPROTO._serialized_end=297
  _APPLIEDCHANGEPROTO._serialized_start=299
  _APPLIEDCHANGEPROTO._serialized_end=407
  _NODEPROTO._serialized_start=410
  _NODEPROTO._serialized_end=594
  _LITERALNODEPROTO._serialized_start=597
  _LITERALNODEPROTO._serialized_end=1214
  _CALLNODEPROTO._serialized_start=1216
  _CALLNODEPROTO._serialized_end=1295
  _LAMBDANODEPROTO._serialized_start=1297
  _LAMBDANODEPROTO._serialized_end=1358
  _KEYQUERYPROTO._serialized_start=1360
  _KEYQUERYPROTO._serialized_end=1388
  _KEYVALUEQUERYPROTO._serialized_start=1390
  _KEYVALUEQUERYPROTO._serialized_end=1438
  _TYPEDQUERYPROTO._serialized_start=1440
  _TYPEDQUERYPROTO._serialized_end=1526
  _QUERIESPROTO._serialized_start=1528
  _QUERIESPROTO._serialized_end=1576
  _ALLQUERYPROTO._serialized_start=1578
  _ALLQUERYPROTO._serialized_end=1593
  _EMPTYQUERYPROTO._serialized_start=1595
  _EMPTYQUERYPROTO._serialized_end=1612
  _CAPPROTO._serialized_start=1614
  _CAPPROTO._serialized_end=1684
  _S2CELLIDSPROTO._serialized_start=1686
  _S2CELLIDSPROTO._serialized_end=1721
  _QUERYPROTO._serialized_start=1724
  _QUERYPROTO._serialized_end=2349
  _FINDFEATUREBYIDREQUESTPROTO._serialized_start=2351
  _FINDFEATUREBYIDREQUESTPROTO._serialized_end=2418
  _FINDFEATUREBYIDRESPONSEPROTO._serialized_start=2420
  _FINDFEATUREBYIDRESPONSEPROTO._serialized_end=2491
  _FINDFEATURESREQUESTPROTO._serialized_start=2493
  _FINDFEATURESREQUESTPROTO._serialized_end=2551
  _FINDFEATURESRESPONSEPROTO._serialized_start=2553
  _FINDFEATURESRESPONSEPROTO._serialized_end=2622
  _MODIFYTAGSREQUESTPROTO._serialized_start=2624
  _MODIFYTAGSREQUESTPROTO._serialized_end=2720
  _MODIFYTAGSBATCHREQUESTPROTO._serialized_start=2722
  _MODIFYTAGSBATCHREQUESTPROTO._serialized_end=2798
  _MODIFYTAGSBATCHRESPONSEPROTO._serialized_start=2800
  _MODIFYTAGSBATCHRESPONSEPROTO._serialized_end=2830
  _EVALUATEREQUESTPROTO._serialized_start=2832
  _EVALUATEREQUESTPROTO._serialized_end=2904
  _EVALUATERESPONSEPROTO._serialized_start=2906
  _EVALUATERESPONSEPROTO._serialized_end=2961
  _B6._serialized_start=2963
  _B6._serialized_end=3034
# @@protoc_insertion_point(module_scope)
