# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: geometry.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0egeometry.proto\x12\x08geometry\"L\n\rPolylineProto\x12$\n\x06points\x18\x01 \x03(\x0b\x32\x14.geometry.PointProto\x12\x15\n\rlength_meters\x18\x02 \x01(\x01\"=\n\x11MultiPolygonProto\x12(\n\x08polygons\x18\x01 \x03(\x0b\x32\x16.geometry.PolygonProto\"2\n\x0cPolygonProto\x12\"\n\x05loops\x18\x01 \x03(\x0b\x32\x13.geometry.LoopProto\"1\n\tLoopProto\x12$\n\x06points\x18\x01 \x03(\x0b\x32\x14.geometry.PointProto\",\n\nPointProto\x12\x0e\n\x06lat_e7\x18\x01 \x01(\x05\x12\x0e\n\x06lng_e7\x18\x02 \x01(\x05\x42\x19Z\x17\x64iagonal.works/b6/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'geometry_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\027diagonal.works/b6/proto'
  _POLYLINEPROTO._serialized_start=28
  _POLYLINEPROTO._serialized_end=104
  _MULTIPOLYGONPROTO._serialized_start=106
  _MULTIPOLYGONPROTO._serialized_end=167
  _POLYGONPROTO._serialized_start=169
  _POLYGONPROTO._serialized_end=219
  _LOOPPROTO._serialized_start=221
  _LOOPPROTO._serialized_end=270
  _POINTPROTO._serialized_start=272
  _POINTPROTO._serialized_end=316
# @@protoc_insertion_point(module_scope)
