# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: features.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import diagonal_b6.geometry_pb2 as geometry__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x66\x65\x61tures.proto\x12\x08\x66\x65\x61tures\x1a\x0egeometry.proto\"&\n\x08TagProto\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"W\n\x0e\x46\x65\x61tureIDProto\x12#\n\x04type\x18\x01 \x01(\x0e\x32\x15.features.FeatureType\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x04\"\x80\x01\n\x11PointFeatureProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\x12 \n\x04tags\x18\x02 \x03(\x0b\x32\x12.features.TagProto\x12#\n\x05point\x18\x03 \x01(\x0b\x32\x14.geometry.PointProto\"\x9f\x01\n\x10PathFeatureProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\x12 \n\x04tags\x18\x02 \x03(\x0b\x32\x12.features.TagProto\x12-\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32\x1b.features.PointFeatureProto\x12\x14\n\x0clengthMeters\x18\x04 \x01(\x01\">\n\x11PathFeaturesProto\x12)\n\x05paths\x18\x01 \x03(\x0b\x32\x1a.features.PathFeatureProto\"\x89\x01\n\x10\x41reaFeatureProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\x12 \n\x04tags\x18\x02 \x03(\x0b\x32\x12.features.TagProto\x12-\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32\x1b.features.PathFeaturesProto\"I\n\x13RelationMemberProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\x12\x0c\n\x04role\x18\x03 \x01(\t\"\x8e\x01\n\x14RelationFeatureProto\x12$\n\x02id\x18\x01 \x01(\x0b\x32\x18.features.FeatureIDProto\x12 \n\x04tags\x18\x02 \x03(\x0b\x32\x12.features.TagProto\x12.\n\x07members\x18\x03 \x03(\x0b\x32\x1d.features.RelationMemberProto\"\xd3\x01\n\x0c\x46\x65\x61tureProto\x12,\n\x05point\x18\x01 \x01(\x0b\x32\x1b.features.PointFeatureProtoH\x00\x12*\n\x04path\x18\x02 \x01(\x0b\x32\x1a.features.PathFeatureProtoH\x00\x12*\n\x04\x61rea\x18\x03 \x01(\x0b\x32\x1a.features.AreaFeatureProtoH\x00\x12\x32\n\x08relation\x18\x04 \x01(\x0b\x32\x1e.features.RelationFeatureProtoH\x00\x42\t\n\x07\x66\x65\x61ture*~\n\x0b\x46\x65\x61tureType\x12\x14\n\x10\x46\x65\x61tureTypePoint\x10\x00\x12\x13\n\x0f\x46\x65\x61tureTypePath\x10\x01\x12\x13\n\x0f\x46\x65\x61tureTypeArea\x10\x02\x12\x17\n\x13\x46\x65\x61tureTypeRelation\x10\x03\x12\x16\n\x12\x46\x65\x61tureTypeInvalid\x10\x04\x42\x19Z\x17\x64iagonal.works/b6/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'features_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\027diagonal.works/b6/proto'
  _FEATURETYPE._serialized_start=1104
  _FEATURETYPE._serialized_end=1230
  _TAGPROTO._serialized_start=44
  _TAGPROTO._serialized_end=82
  _FEATUREIDPROTO._serialized_start=84
  _FEATUREIDPROTO._serialized_end=171
  _POINTFEATUREPROTO._serialized_start=174
  _POINTFEATUREPROTO._serialized_end=302
  _PATHFEATUREPROTO._serialized_start=305
  _PATHFEATUREPROTO._serialized_end=464
  _PATHFEATURESPROTO._serialized_start=466
  _PATHFEATURESPROTO._serialized_end=528
  _AREAFEATUREPROTO._serialized_start=531
  _AREAFEATUREPROTO._serialized_end=668
  _RELATIONMEMBERPROTO._serialized_start=670
  _RELATIONMEMBERPROTO._serialized_end=743
  _RELATIONFEATUREPROTO._serialized_start=746
  _RELATIONFEATUREPROTO._serialized_end=888
  _FEATUREPROTO._serialized_start=891
  _FEATUREPROTO._serialized_end=1102
# @@protoc_insertion_point(module_scope)
